// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef	OLENA_CORE_DPOINT1D_HXX
#define OLENA_CORE_DPOINT1D_HXX

#include <iostream>
#include "core/point1d.hh"


namespace oln {


  inline
  dpoint1d::dpoint1d() :
    _col(0)
  {
  }

  inline
  dpoint1d::dpoint1d(coord col) :
    _col(col)
  {
  }

  inline
  dpoint1d::dpoint1d(const point1d& p) :
    _col(p.col())
  {
  }

  inline coord
  dpoint1d::col() const
  {
    return _col;
  }

  inline coord&
  dpoint1d::col()
  {
    return _col;
  }

  inline bool
  dpoint1d::operator==(const dpoint1d& dp) const
  {
    return _col == dp.col();
  }

  inline bool
  dpoint1d::operator!=(const dpoint1d& dp) const
  {
    return _col != dp.col();
  }

  inline dpoint1d
  dpoint1d::operator-() const
  {
    dpoint1d dp(-_col);
    return dp;
  }

  inline dpoint1d
  dpoint1d::operator+(const dpoint1d& dp) const
  {
    dpoint1d tmp = *this;
    tmp += dp;
    return tmp;
  }

  inline dpoint1d
  dpoint1d::operator-(const dpoint1d& dp) const
  {
    dpoint1d tmp = *this;
    tmp -= dp;
    return tmp;
  }

  inline dpoint1d&
  dpoint1d::operator+=(const dpoint1d& dp)
  {
    _col += dp.col();
    return *this;
  }

  inline dpoint1d&
  dpoint1d::operator-=(const dpoint1d& dp)
  {
    _col -= dp.col();
    return *this;
  }

  inline bool
  dpoint1d::is_centered(void) const
  {
    return _col == 0;
  }

} // end of oln


inline std::ostream&
operator<<(std::ostream& o, const oln::dpoint1d& dp)
{
  return o << '(' << dp.col() << ')';
}


#endif // ! OLENA_CORE_DPOINT1D_HXX
