// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef	OLENA_CORE_DPOINT2D_HXX
#define OLENA_CORE_DPOINT2D_HXX

#include <iostream>
#include "core/point2d.hh"


namespace oln {


  inline
  dpoint2d::dpoint2d() :
    _row(0),
    _col(0)
  {
  }

  inline
  dpoint2d::dpoint2d(coord row, coord col) :
    _row(row),
    _col(col)
  {
  }

  inline
  dpoint2d::dpoint2d(const point2d& p) :
    _row(p.row()),
    _col(p.col())
  {
  }

  inline coord
  dpoint2d::row() const
  {
    return _row;
  }

  inline coord&
  dpoint2d::row()
  {
    return _row;
  }

  inline coord
  dpoint2d::col() const
  {
    return _col;
  }

  inline coord&
  dpoint2d::col()
  {
    return _col;
  }

  inline bool
  dpoint2d::operator==(const dpoint2d& dp) const
  {
    return _row == dp.row() && _col == dp.col();
  }

  inline bool
  dpoint2d::operator!=(const dpoint2d& dp) const
  {
    return _row != dp.row() || _col != dp.col();
  }

  inline dpoint2d
  dpoint2d::operator-() const
  {
    dpoint2d dp(-_row, -_col);
    return dp;
  }

  inline dpoint2d
  dpoint2d::operator+(const dpoint2d& dp) const
  {
    dpoint2d tmp = *this;
    tmp += dp;
    return tmp;
  }

  inline dpoint2d
  dpoint2d::operator-(const dpoint2d& dp) const
  {
    dpoint2d tmp = *this;
    tmp -= dp;
    return tmp;
  }

  inline dpoint2d&
  dpoint2d::operator+=(const dpoint2d& dp)
  {
    _row += dp.row();
    _col += dp.col();
    return *this;
  }

  inline dpoint2d&
  dpoint2d::operator-=(const dpoint2d& dp)
  {
    _row -= dp.row();
    _col -= dp.col();
    return *this;
  }

  inline bool
  dpoint2d::is_centered(void) const
  {
    return _row == 0 && _col == 0;
  }

} // end of oln


inline std::ostream&
operator<<(std::ostream& o, const oln::dpoint2d& dp)
{
  return o << '(' << dp.row() << ',' << dp.col() << ')';
}


#endif // ! OLENA_CORE_DPOINT2D_HXX
