// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef OLENA_CORE_DPOINT3D_HH
# define OLENA_CORE_DPOINT3D_HH

# include <iostream>
# include "core/coord.hh"
# include "core/dpoint.hh"
// # include "value/built_in.hh"


namespace oln {


  // fwd decl
  class point3d;
  template <unsigned N, class T> class vec;


  class dpoint3d : public dpoint< dpoint3d >
  {
  public:
    typedef type::bottom inferior;

    enum { dim = 3 };

    dpoint3d();
    dpoint3d(coord slice, coord row, coord col);

    explicit dpoint3d(const point3d& p);

//     typedef vec< 3, internal::oln_for_C<coord>::ret > vec_t;
//     operator vec_t() const; // impl. in value/vec.hh
    // FIXME: can operator above be explicit?

    coord slice() const;
    coord& slice();
    coord row() const;
    coord& row();
    coord col() const;
    coord& col();

    bool operator==(const dpoint3d& dp) const;
    bool operator!=(const dpoint3d& dp) const;

    dpoint3d operator+(const dpoint3d& dp) const;
    dpoint3d operator-() const;
    dpoint3d operator-(const dpoint3d& dp) const;
    dpoint3d& operator+=(const dpoint3d& dp);
    dpoint3d& operator-=(const dpoint3d& dp);

    bool is_centered(void) const;

    static std::string name() { return "dpoint3d"; }

  private:

    coord _slice;
    coord _row;
    coord _col;
  };

  _DPointForDim(3, dpoint3d);

  namespace internal
  {

    template<>
    struct default_less<dpoint3d>
    {
      bool operator()(const dpoint3d& lhs, const dpoint3d& rhs) const
      {
	return (lhs.slice() < rhs.slice()
		|| (lhs.slice() == rhs.slice()
		    && (lhs.row() < rhs.row()
			|| (lhs.row() == rhs.row()
			    && lhs.col() < rhs.col()))));
      }
    };

  } // end of internal


} // end of oln


inline std::ostream&
operator<<(std::ostream& o, const oln::dpoint3d& p);


#include "core/dpoint3d.hxx"


#endif // ! OLENA_CORE_DPOINT3D_HH
