// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef	OLENA_CORE_DPOINT3D_HXX
#define OLENA_CORE_DPOINT3D_HXX

#include <iostream>
#include "core/point3d.hh"


namespace oln {


  inline
  dpoint3d::dpoint3d() :
    _slice(0),
    _row(0),
    _col(0)
  {
  }

  inline
  dpoint3d::dpoint3d(coord slice, coord row, coord col) :
    _slice(slice),
    _row(row),
    _col(col)
  {
  }

  inline
  dpoint3d::dpoint3d(const point3d& p) :
    _slice(p.slice()),
    _row(p.row()),
    _col(p.col())
  {
  }

  inline coord
  dpoint3d::slice() const
  {
    return _slice;
  }

  inline coord&
  dpoint3d::slice()
  {
    return _slice;
  }

  inline coord
  dpoint3d::row() const
  {
    return _row;
  }

  inline coord&
  dpoint3d::row()
  {
    return _row;
  }

  inline coord
  dpoint3d::col() const
  {
    return _col;
  }

  inline coord&
  dpoint3d::col()
  {
    return _col;
  }

  inline bool
  dpoint3d::operator==(const dpoint3d& dp) const
  {
    return (_slice == dp.slice()
	    && _row == dp.row()
	    && _col == dp.col());
  }

  inline bool
  dpoint3d::operator!=(const dpoint3d& dp) const
  {
    return (_slice != dp.slice()
	    || _row != dp.row()
	    || _col != dp.col());
  }

  inline dpoint3d
  dpoint3d::operator-() const
  {
    dpoint3d dp(-_slice, -_row, -_col);
    return dp;
  }

  inline dpoint3d
  dpoint3d::operator+(const dpoint3d& dp) const
  {
    dpoint3d tmp = *this;
    tmp += dp;
    return tmp;
  }

  inline dpoint3d
  dpoint3d::operator-(const dpoint3d& dp) const
  {
    dpoint3d tmp = *this;
    tmp -= dp;
    return tmp;
  }

  inline dpoint3d&
  dpoint3d::operator+=(const dpoint3d& dp)
  {
    _slice += dp.slice();
    _row += dp.row();
    _col += dp.col();
    return *this;
  }

  inline dpoint3d&
  dpoint3d::operator-=(const dpoint3d& dp)
  {
    _slice -= dp.slice();
    _row -= dp.row();
    _col -= dp.col();
    return *this;
  }

  inline bool
  dpoint3d::is_centered(void) const
  {
    return _slice == 0 && _row == 0 && _col == 0;
  }

} // end of oln


inline std::ostream&
operator<<(std::ostream& o, const oln::dpoint3d& dp)
{
  return o << '(' << dp.slice() << ',' << dp.row() << ',' << dp.col() << ')';
}


#endif // ! OLENA_CORE_DPOINT3D_HXX
