// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef OLENA_CORE_INTERNAL_ITER3D_HH
# define OLENA_CORE_INTERNAL_ITER3D_HH

# include "core/contract.hh"
# include "core/type.hh"
# include "core/coord.hh"
# include "core/point3d.hh"
# include "core/dpoint3d.hh"
# include "core/image3d.hh"
# include "core/objs.hh"


namespace oln {

  namespace internal {


    template<class Inferior = type::bottom>
    class _iter3d : public type::any< _iter3d<Inferior> >
    {
    public:
      typedef Inferior inferior;
//       typedef _iter3d<Inferior> self;
//       typedef typename type::exact<self>::ret exact;

      const point3d& point_ref() const
      {
	return _p;
      }

      bool operator==(const point3d& p) const
      {
	return _p == p;
      }

      bool operator!=(const point3d& p) const
      {
	return _p != p;
      }

      point3d operator+(const dpoint3d& dp) const
      {
	precondition(*this != end);
	return _p + dp;
      }

      point3d operator-(const dpoint3d& dp) const
      {
	precondition(*this != end);
	return _p - dp;
      }

      operator point3d() const
      {
	precondition(*this != end);
	invariant(_p.slice() >=0
		  && _p.slice() < _nslices
		  && _p.row() >= 0
		  && _p.row() < _nrows
		  && _p.col() >= 0
		  && _p.col() < _ncols);
	return _p;
      }

      coord slice() const
      {
	return _p.slice();
      }

      coord row() const
      {
	return _p.row();
      }

      coord col() const
      {
	return _p.col();
      }

      // deferred methods are:
      //
      //   void _goto_begin();
      //   void _goto_end();
      //   bool _is_at_end() const;
      //   void _goto_next();

      internal::_begin operator=(internal::_begin b)
      {
	to_exact(this)->_goto_begin();
	return b;
      }

      internal::_end operator=(internal::_end e)
      {
	to_exact(this)->_goto_end();
	return e;
      }

      bool operator==(internal::_end) const
      {
	return to_exact(this)->_is_at_end();
      }

      void operator++()
      {
	precondition(*this != end);
	to_exact(this)->_goto_next();
// 	return to_exact(*this);
      }

      // deduced methods:

      bool operator!=(internal::_end e) const
      {
	return ! this->operator==(e);
      }

//       typename type::exact<self>::ret operator++(int)
//       {
// 	precondition(*this != end);
// 	typename type::exact<self>::ret tmp = to_exact(*this);
// 	this->operator++();
// 	return tmp;
//       }

      static std::string name() { return std::string("_iter3d<") + Inferior::name() + ">"; }

    protected:

      point3d _p;
      const coord _nslices;
      const coord _nrows;
      const coord _ncols;

      _iter3d(const image3d_size& size) :
	_nslices(size.nslices()),
	_nrows(size.nrows()),
	_ncols(size.ncols())
      {
	precondition(size.nslices() > 0
		     && size.nrows() > 0
		     && size.ncols() > 0);
	to_exact(this)->_goto_begin();
      }
    };


  } // end of internal

} // end of oln


#endif // ! OLENA_CORE_INTERNAL_ITER3D_HH
